/*
	power_mng.c --- power management functions
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <stdio.h>

#include "user_define.h"

/* -------- global var. for power_mng -------- */
int PM_MODE;
int SLEEP_TIMER;
unsigned int POWCHK_COUNT;		// counter to check from USB-UART power

/*
 *  Power management start as RUN0 mode
 */
void init_pmmode()
{
    set_pmmode(PM_MODE_RUN0);
    any_activity();
    POWCHK_COUNT = 0;
}
/*
 * check external power supply existance status
 * using RxD signal level via P1.5 input (with pull-down R setting)
 */
void check_powstat()
{
	if (poll_sw1())	set_pmmode(PM_MODE_RUN1);

	if (poll_rxd()){		// P1.5 input is High
		if (PM_MODE <= PM_MODE_LOW){	// sleeping now
			POWCHK_COUNT = 1;
			set_pmmode(PM_MODE_RUN1);	// battery operation now
		}
		else if (PM_MODE == PM_MODE_RUN1){
			if (POWCHK_COUNT > 0){
				POWCHK_COUNT++;
				if (POWCHK_COUNT >= 5){
					set_pmmode(PM_MODE_RUN0);	// power from USB-UART I/F exists
					POWCHK_COUNT = 0;
				}
			}
		}
	} else {				// P1.5 input is Low
		if (PM_MODE == PM_MODE_RUN0){
			set_pmmode(PM_MODE_RUN1);	// RUN0->RUN1,battery operation now
		}
		POWCHK_COUNT = 0;
	}
}

/*
 * set power management mode
 */
void set_pmmode(unsigned char mode)
{
	//int prev_mode;

	if (mode == PM_MODE_RUN0) uart_sendstr("(PM:RUN0)\r\n"); //DEBUG
	if (mode == PM_MODE_RUN1) uart_sendstr("(PM:RUN1)\r\n"); //DEBUG
	if (mode == PM_MODE_LOW) uart_sendstr("(PM:LOW)\r\n"); //DEBUG

	//prev_mode = PM_MODE;
	PM_MODE = mode;
	if (PM_MODE == PM_MODE_RUN0){	// full run now
		//if (prev_mode == PM_MODE_LOW)		// return from _LOW mode
		init_uart();
		//P1OUT |= BIT0;	// LED1 On
	}
	else {
		//P1OUT &= ~BIT0;	// LED1 Off
	}

	if (PM_MODE == PM_MODE_LOW){	// enter to sleep
		close_uart();
	}
}
/*
 * return current power management mode
 */
unsigned char cur_pmmode()
{
	return PM_MODE;
}

/*
 * any activity happened .. sleep timer reset
 */
void any_activity()
{
	SLEEP_TIMER = SLEEP_TIMER_LIMIT;	// reset to full
	//check_powstat();
}

/*
 * tick from WDT
 */
void wdt_tick_pm()
{
	int w;
	int z;	//DEBUG

	if ((PM_MODE <= 0) || (PM_MODE >20)){		//DEBUG
		fatal_error(1, PM_MODE, "wdt_tick_pm");	// DEBUG
		set_pmmode(PM_MODE_RUN0);	//DEBUG
	}
	else {
		P1OUT |= BIT0;	// LED1 On	//DEBUG
		//if (PM_MODE > PM_MODE_LOW) z=10000; else z=2000;	//DEBUG
		z = PM_MODE * 1000;		//DEBUG
		for (w=1; w<z; w++)	;		//DEBUG
		P1OUT &= ~BIT0;	// LED1 Off	//DEBUG
	}

	check_powstat();
	if (PM_MODE == PM_MODE_RUN1){	// battery operation now
		//P1OUT |= BIT0;	// LED1 On
		if (POWCHK_COUNT == 0){		// now, not checking wakeup ext.power
			SLEEP_TIMER--;
			if (SLEEP_TIMER <= 0){
				set_pmmode(PM_MODE_LOW);
			}
		}

		//for (w=0; w<1500; w++)
		//	;
		//P1OUT &= ~BIT0;	// LED1 Off
	}
	else if (PM_MODE == PM_MODE_RUN0){	// full run now
		any_activity();
	}

	if (PM_MODE == PM_MODE_LOW){
		PM_MODE = PM_MODE_SLP;	//DEBUG
		//_BIS_SR(LPM3_bits + GIE);	// Enter LPM3
	}

}


/* ==============
 * main loop --- infinite loop and dispatch any functions
 * ==============
 */
void loop_dispatch()
{
	int m;


	while (1){
		for (m=0; m<12; m++){		//DEBUG
			if ((mon_day_tbl[m] < 0) || (mon_day_tbl[m] > 31))
				fatal_error(1, m, "dispatch, mon_day_tbl");
		}

		if (RTC_TICK){			// wwakeup caused by WDT
			RTC_TICK = FALSE;
			wdt_tick_record();	// for recording
			wdt_tick_pm();		// for power_mng
		}
		session();

		if (PM_MODE == PM_MODE_SLP){
			_BIS_SR(LPM3_bits + GIE);	// Enter LPM3
		}
	}	// while (1)
}


/*
 * error trap  (through, display only)
 */
void show_error(int code, int param1, char *mes)
{
	char str[10];
	uart_sendstr("# ERROR: ");
	uart_sendstr(mes);
	numstr(code, str);		uart_sendstr(" code=");	uart_sendstr(str);
	hexstr(code, str);		uart_sendstr(" (0x");	uart_sendstr(str);
	uart_sendstr(")");
	numstr(param1, str);	uart_sendstr(" param1=");	uart_sendstr(str);
	hexstr(param1, str);	uart_sendstr(" (0x");	uart_sendstr(str);
	uart_sendstr(") \r\n");
}

/*
 * error trap
 */
void fatal_error(int code, int param1, char *mes)
{
	show_error(code, param1, mes);
	while (1)	;	//HALT
	//return;
}
