/*
	rec_mng.c --- Recording temperature data management function
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>

#include "user_define.h"

/* -------- global var. rec_mng -------- */
int REC_MODE;
unsigned int REC_ITEMS;
unsigned char REC_CONFIG_MODE;
unsigned long REC_START_TIME;

unsigned int REC_PERIOD_SEC[4];
unsigned int REC_RTC_MASK[4];
unsigned int REC_RTC_SHIFT[4];
unsigned int REC_START_ADR_PTR, REC_CUR_ADR_PTR;
unsigned int REC_TIME_OFS;

/*
 * initialize vars, tables
 */
void init_rec(void)
{
	int j;

	REC_MODE = REC_MODE_STOP;
	REC_START_ADR_PTR = 0x0000;
	REC_CUR_ADR_PTR = 0x0000;
	REC_ITEMS = 0;
	REC_CONFIG_MODE = 0;

	REC_PERIOD_SEC[0] = REC_CFG_PERI_MODE_0;	// rec config 0
	REC_PERIOD_SEC[1] = REC_CFG_PERI_MODE_1;	// rec config 1
	REC_PERIOD_SEC[2] = REC_CFG_PERI_MODE_2;	// rec config 2
	REC_PERIOD_SEC[3] = REC_CFG_PERI_MODE_3;	// rec config 3

	REC_RTC_SHIFT[0] = REC_CFG_SFT_MODE_0;	// rec config 0
	REC_RTC_SHIFT[1] = REC_CFG_SFT_MODE_1;	// rec config 1
	REC_RTC_SHIFT[2] = REC_CFG_SFT_MODE_2;	// rec config 2
	REC_RTC_SHIFT[3] = REC_CFG_SFT_MODE_3;	// rec config 3

	for (j=0; j<4; j++){
		REC_RTC_MASK[j] = REC_PERIOD_SEC[j] - 1;	// ex) period 16s -> 0x000F
	}
}


/*
 * set recording configuration
 */
int set_rec_config(int rec_cfg)
{
	REC_CONFIG_MODE = rec_cfg;
	return rec_cfg;
}
/*
 * set recording mode (run/stop flag)
 */
int set_rec_mode(int new_rec_mode)
{
	int prev_mode;

	prev_mode = REC_MODE;

	if (new_rec_mode != prev_mode){
		REC_MODE = new_rec_mode;
		if (new_rec_mode == REC_MODE_RUN)	// stop -> run
			record_open();
		else 				// run -> stop
			record_close();
	}
	return REC_MODE;
}

/*
 * set address pointer
 */
BOOL set_adr_ptr(unsigned int new_ptr)
{
	if (new_ptr >= REC_MEM_SIZE)	return FALSE;		// bad range

	REC_START_ADR_PTR = REC_CUR_ADR_PTR = (new_ptr & 0xFFFE);
	return TRUE;
}

/*
 *  start recording (logging) ... write information header to eeprom
 */
void record_open()
{
	unsigned long int rtc_start_time;
	unsigned char data[8];
	int ret;

	REC_START_ADR_PTR = REC_CUR_ADR_PTR;	// recoed start address
	REC_ITEMS = 0;
	REC_TIME_OFS = 0;

	if ((RTC_TIME_COUNTER & REC_RTC_MASK[REC_CONFIG_MODE]) == 0){
		rtc_start_time = RTC_TIME_COUNTER;
	}
	else {
		rtc_start_time = (RTC_TIME_COUNTER + REC_PERIOD_SEC[REC_CONFIG_MODE])
	    	         & (~(unsigned long int)(REC_RTC_MASK[REC_CONFIG_MODE]));

	}

	data[0] = REC_HEADER_MARK1;
	data[1] = REC_HEADER_MARK2 + REC_CONFIG_MODE;
	data[2] = (unsigned char)(rtc_start_time & 0xFF);
	data[3] = (unsigned char)((rtc_start_time >> 8) & 0xFF);
	data[4] = (unsigned char)((rtc_start_time >> 16) & 0xFF);
	data[5] = (unsigned char)((rtc_start_time >> 24) & 0xFF);
	data[6] = 0xFF;
	data[7] = 0xFF;
	ret = eeprom_wr(REC_CUR_ADR_PTR, data, 8);	// write info. header to eeprom
	if (ret < 0)
		show_error(ret, 0, "eeprom_wr in rec_open");
	REC_CUR_ADR_PTR += 8;
}
/*
 *  stop recording (logging) ... close
 */
void record_close()
{
	unsigned char data[2];
	int ret;

	data[0] = (unsigned char)(REC_ITEMS & 0xFF);
	data[1] = (unsigned char)((REC_ITEMS >> 8) & 0xFF);
	ret = eeprom_wr(REC_START_ADR_PTR + 6, data, 2);	// write items into header area
	if (ret < 0)
		show_error(ret, 0, "eeprom_wr in rec_close");
}

/*
 *  list records and select by user
 */
void record_list_select()
{
	unsigned int adr_list[REC_LIST_MAX];	// currently, 32
	unsigned int adr_ptr, items, next_adr;
	int idx;
	unsigned long int timestamp;
	unsigned char data[10];
	char str[10];

	uart_sendstr("Listing of records (max 32 record):\r\n");
	uart_sendstr("searching...\r\n");
	uart_sendstr(" No. Addr.  Start date & time   Cfg Items  Next addr\r\n");
	uart_sendstr("---- ------ ------------------- --- ------ ------\r\n");

	adr_ptr = 0;
	idx = 0;

	while ((idx < REC_LIST_MAX) && (adr_ptr < REC_MEM_SIZE)){
		eeprom_rd(adr_ptr + 1, data, 1);
		if ((data[0] & 0xFC) == REC_HEADER_MARK2){		// 0xEx
			eeprom_rd(adr_ptr, data, 8);
			if (data[0] == REC_HEADER_MARK1){		// found MARKER1 and 2
				eeprom_rd(adr_ptr, data, 8);
				timestamp = (unsigned long)data[2];
				timestamp |= ((unsigned long)(data[3])) << 8;
				timestamp |= ((unsigned long)(data[4])) << 16;
				timestamp |= ((unsigned long)(data[5])) << 24;
				items = (((unsigned int)data[7]) << 8) + data[6];
				uart_sendstr(" ");
				numstr(idx + 1, str);
				uart_sendstr(&str[3]);		uart_sendstr(": 0x");
				hexstr(adr_ptr, str);
				uart_sendstr(str);		uart_sendstr(" ");
				show_date_time(rtc_count2ymd(timestamp));
				uart_sendstr(" ");
				numstr(data[1] & 0x03, str);		// config #
				uart_sendstr(" ");	uart_sendstr(&str[4]);		uart_sendstr("  ");
				if (items == 0xFFFF){
					uart_sendstr("(RUN)");
				} else {
					numstr(items, str);	uart_sendstr(str);
				}
				next_adr = adr_ptr + 8 + 4 * items;
				hexstr(next_adr, str);
				uart_sendstr("  0x");	uart_sendstr(str);
				uart_sendstr("\r\n");

				adr_list[idx++] = adr_ptr;
				adr_ptr += 8;
			}
			else {}  // skip
		}
		else {
			adr_ptr += 2;		// skip
		}
	} // while

}

/*
 * from WDT interval call
 */
void wdt_tick_record()
{
	unsigned int temp_rec;
	unsigned char data[4];
	//char str[10];	//DEBUG

	if (REC_MODE == REC_MODE_STOP)	return;

	// recording mode now
	// if it is not time to record, quit
	if (RTC_TIME_COUNTER & (unsigned long)REC_RTC_MASK[REC_CONFIG_MODE])	return;

	// should measure temp. and record
	//temp_test1();
	//__bis_SR_register(GIE);	// Enable Interrupt, because it is still disable
	temp_rec = get_temperature();

	//hexstr(temp_rec, str);				//DEBUG
	//uart_sendstr("temp. = 0x");		//DEBUG
	//uart_sendstr(str);				//DEBUG


	if (REC_ITEMS == 0){	// the first record
		REC_START_TIME = RTC_TIME_COUNTER;	// record start time
	}
	data[0] = (unsigned char)(REC_TIME_OFS & 0x00FF);		// lower
	data[1] = (unsigned char)(REC_TIME_OFS >> 8);			// upper
	data[2] = (unsigned char)(temp_rec & 0x00FF);		// lower
	data[3] = (unsigned char)(temp_rec >> 8);			// upper
	eeprom_wr(REC_CUR_ADR_PTR, data, 4);
    //uart_sendstr(" rec done (debug)\r\n"); //DEBUG

	REC_CUR_ADR_PTR = (REC_CUR_ADR_PTR + 4) & (REC_MEM_SIZE - 1);	// & 0x7FFF
	REC_ITEMS++;
	REC_TIME_OFS++;

}


/*
 * show current status & config
 */
void show_current_rec_status()
{
	char str[10];
	uart_sendstr("Logging status:\r\n");
	uart_sendstr(" record config No = ");
	uart_sendchar('0' + REC_CONFIG_MODE);
	uart_sendstr("\r\n");
	uart_sendstr(" record start mem address = 0x");
	hexstr(REC_START_ADR_PTR, str);
	uart_sendstr(str);	uart_sendstr("\r\n");
	uart_sendstr(" rec. current mem address = 0x");
	hexstr(REC_CUR_ADR_PTR, str);
	uart_sendstr(str);	uart_sendstr("\r\n");
}
/*
 * dump the recorded data
 */
void dump_record(unsigned int start_addr, unsigned int items)
{
	unsigned int addr_ptr, cnt, cfg;
	unsigned int temp_raw;
	unsigned long int rec_rtc_time, timestamp, t;
	unsigned char data[8];
	char str[40];
	DATETYPE date;

	addr_ptr = start_addr;
	eeprom_rd(addr_ptr, data, 8);
	if (data[0] != REC_HEADER_MARK1)
		uart_sendstr("Caution! -- illegal header\r\n");
	cfg = data[1] & 0x03;
	timestamp = (unsigned long)data[2];
	timestamp |= ((unsigned long)(data[3])) << 8;
	timestamp |= ((unsigned long)(data[4])) << 16;
	timestamp |= ((unsigned long)(data[5])) << 24;
	if (items == 0)
		items = (((unsigned int)data[7]) << 8) + data[6];
	addr_ptr += 8;

	for (cnt = 0; cnt < items; cnt++){
		eeprom_rd(addr_ptr, data, 4);
		t = ((unsigned long)data[1] << 8) + data[0];
		rec_rtc_time = timestamp + (t * REC_PERIOD_SEC[cfg]);
		date = rtc_count2ymd(rec_rtc_time);		// convert to y/m/d h:m:s
		temp_raw = ((unsigned int)data[3] << 8) + data[2];
		raw2degree(temp_raw, str);
		show_date_time(date);
		//uart_sendstr(", ");
		uart_sendchar(TAB);
		uart_sendstr(str);
		uart_sendstr("\r\n");
		addr_ptr += 4;
	}
}
/*
 * show config options detail (session help message)
 */
void show_config_options()
{
	int j;
	char str[10];
	for (j=0; j<4; j++){
		uart_sendstr("  Config No = ");
		uart_sendchar('0' + j);
		uart_sendstr(" : logging period = ");
		numstr(REC_PERIOD_SEC[j], str);
		uart_sendstr(&str[2]);
		uart_sendstr(" second\r\n");
	}
}

/*
 * erase all memory (fill all area by 0xFF)
 */
void erase_all_memory(void)
{
	int j, err_cnt;
	int err;
	unsigned char buff[WORK_BUFFSIZE];		// 128
	unsigned int adr_ptr;
	char str[8];
	BOOL ok;

	adr_ptr = 0;
	err_cnt = 0;
	uart_sendstr("Erase all memory --- fill all area as 0xFF...\r\n");
	while (adr_ptr < REC_MEM_SIZE){
		ok = TRUE;
		for (j=0; j < WORK_BUFFSIZE; j++) buff[j] = 0xFF;
		err = eeprom_wr(adr_ptr, buff, WORK_BUFFSIZE);	// erase
		if (err <= 0)	fatal_error(err, adr_ptr, "erase eeprom_Wr");
		err = eeprom_rd(adr_ptr, buff, WORK_BUFFSIZE);	// check
		if (err <= 0)	;//show_error(err, adr_ptr, "erase eeprom_rd");
		for (j=0; j < WORK_BUFFSIZE; j++)
			if (buff[j] != 0xFF){	ok = FALSE;	err_cnt++;	}
		if (ok)	uart_sendchar('.');
		else	uart_sendchar('x');
		adr_ptr +=  WORK_BUFFSIZE;
	}
	if (err_cnt == 0){
		uart_sendstr("\r\nErase complete successfully\r\n");
	} else {
		uart_sendstr("Erase error count: ");
		numstr(err_cnt, str);
		uart_sendstr(str);
		uart_sendstr("bytes\r\n");
	}
}

