/*
	rtc.c --- Real Time Clock fuunctionality
*/

#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include <math.h>

#include "user_define.h"



/* -------- global var. for rtc -------- */
unsigned long RTC_TIME_COUNTER;
BOOL RTC_TICK;		// marker, updated by WDT interval interrupt
BOOL RTC_FLG1;		// DEBUG,  WDT updated the counter

int mon_day_tbl[] = {31,28,31,30,31,30,31,31,30,31,30,31};

/*
 * reset RTC master counter var.
 */
void rtc_reset()
{
	RTC_TIME_COUNTER = 0;
	RTC_TICK = FALSE;
}

/*
 * set RTC master counter var. by y/m/d/ h:m:s
 *   return: different from current RTC_TIME_COUNTER
 */
int set_rtc_counter(int year, char mon, char day, char hour, char min, char sec)
{
	unsigned long int new_cnt, new_days;
	BOOL wr_ok;
	int m, leap;
	int diff;

	/*for (m=0; m<12; m++){		//DEBUG
		if ((mon_day_tbl[m] < 0) || (mon_day_tbl[m] > 31))
			fatal_error(1, m, "mon_day_tbl");
	}*/

	wr_ok = FALSE;
	leap = 0;
	diff = 0;

	if (((year >= 2000) && (year < 2100)) &&
		((mon >= 1) && (mon <= 12)) &&
		((day >= 1) && (day <= 31)) &&
		((hour >= 0) && (hour < 24)) &&
		((min >= 0) && (min < 60))  )
			wr_ok = TRUE;

	if (wr_ok){
		new_days = 365UL * (year - 2000);
		//new_days += floor((year - 2000 - 1) / 4) + 1;
		new_days += ((year - 2000 - 1) / 4) + 1;
		if (mon >= 3){
			if ((year % 4) == 0){ leap = 1; }
		}
		new_days += leap;
		for (m=0; m<mon-1; m++){
			new_days += mon_day_tbl[m];
		}
		new_days += day - 1;
		new_cnt = 86400L * new_days;
		//rtc_reset();
		new_cnt += 3600L * hour + 60L * min + sec;
		diff = (int)(new_cnt - RTC_TIME_COUNTER);
		RTC_TIME_COUNTER = new_cnt;
	}
	return diff;
}

/*
 * convert RTC master counter value (32bit) to y/m/d hh:mm:ss
 */
DATETYPE rtc_count2ymd(unsigned long count)
{
	DATETYPE date;
	unsigned long days,secs;
	int m, leap, mday;

	leap = 0;
//	days = floor(count / 86400L);	// 86400 = 24*3600 [sec]
	days = count / 86400L;	// 86400 = 24*3600 [sec]
	secs = count % 86400L;

//	date.year = 4 * floor(days / 1461L);
	date.year = 4 * (days / 1461L);
	days = days % 1461;
	if (days < 366){
		// -> go to month/date calc.
	} else {
		days--;
		//date.year += floor(days / 365);
		date.year += (days / 365);
		days = days % 365;
	}
	date.year += 2000;
	if ((date.year % 4) == 0){ leap = 1; }

	for (m=0; m<12; m++){
		mday = mon_day_tbl[m];
		if ((leap > 0) && (m == 1)){ mday++; }
		if (days < mday){ break; }
		else {
			days -= mday;
		}
	}
	date.mon = m + 1;
	date.day = days + 1;

//	date.hour = floor(secs / 3600L);
	date.hour = (secs / 3600L);
//	date.min = floor((secs % 3600L) / 60);
	date.min = ((secs % 3600L) / 60);
	date.sec = secs % 60;

	return date;
}

/*
 * show DATETYPE data to yyyy/mm/dd hh:mm:ss
 */
void show_date_time(DATETYPE date)
{
	char str[8];
	numstr(date.year, str);	uart_sendstr(&str[1]); uart_sendchar('/');
	numstr(date.mon, str);	uart_sendstr(&str[3]); uart_sendchar('/');
	numstr(date.day, str);	uart_sendstr(&str[3]); uart_sendchar(' ');
	numstr(date.hour, str);	uart_sendstr(&str[3]); uart_sendchar(':');
	numstr(date.min, str);	uart_sendstr(&str[3]); uart_sendchar(':');
	numstr(date.sec, str);	uart_sendstr(&str[3]);

}

/*
 * read XCAP setting
 *   return:  0=1pF, 1=6pF, 2=10pF, 3=12.5pF
 */
int get_xcap_setting()
{
	return ((BCSCTL3 & 0x0c) >> 2);
}
/*
 * change XCAP setting
 *   input:  sel: 0=1pF, 1=6pF, 2=10pF, 3=12.5pF
 */
void set_xcap_setting(int sel)
{
	unsigned char r;
	sel &= 0x03;
	r = BCSCTL3;
	r = (r & 0xf0) | (sel << 2);
	BCSCTL3 = r;
}
