/*
 *================================================================
 * Temperature Logger using MSP430G2553
 *   This program access I2C bus using USCI_B0 as I2C master port,
 *   and can be operated by user terminal via USCI_A0 UART port.
 *
 *   Copyright (C) 2013-2014  espilab (espi laboratory)
 *
 *   Ver.   Date       Comment
 *  ------ ---------- -------------------------------
 *   1.0   2014-05-01 first release
 *================================================================
 */

/*
	user_define.h
*/

#ifndef _USER_DEFS_
#define _USER_DEFS_


/* ======== general definition ======== */
#define	BOOL	unsigned char
#define TRUE	1
#define FALSE	0

/* ================================================
 * uart.c  related defines,vars,functions
 * ================================================ */
/* -------- define for serial port (UART) -------- */
#define CTRLC	0x03
#define BEL   0x07
#define BS    0x08
#define TAB	  0x09
#define SPC   0x20
#define CR    0x0d
#define LF    0x0a
#define ESC	  0x1B

#define UART_RXBUFMAX  40

/* -------- global var. for serial port -------- */
extern unsigned char UART_RXDBUF[UART_RXBUFMAX];
extern int UART_RCV_PTR, UART_READ_PTR, UART_RCV_LEN;

/* -------- functions for serial port -------- */
void init_uart(void);
void close_uart(void);
int uart_sendchar(unsigned char data);
void uart_sendstr(char *s);
unsigned char uart_getchar();
unsigned int recv_len();


/* ================================================
 * i2c.c  related defines,vars,functions
 * ================================================ */
/* -------- define for i2c -------- */
#define I2C_BUFSIZE	64
#define E_OK		0
#define E_NACK		(-1)
#define E_TMOUT		(-2)
#define E_NOTFND	(-3)
#define E_BUSY		(-4)
#define E_NOCHANGE	(-256)		// 0xFF00

/* -------- global var. for i2c -------- */
extern unsigned char I2C_SLAVE_ADR;
extern unsigned char I2C_BUF_W[I2C_BUFSIZE];
extern unsigned char I2C_BUF_R[I2C_BUFSIZE];
extern int I2C_TxByteCnt, I2C_RxByteCnt;
extern unsigned char *I2C_TxDataPtr, *I2C_RxDataPtr;
extern int I2C_TxComplete;

/* -------- functions for i2c -------- */
int i2c_send(unsigned int slave_addr, unsigned char *txdata, int txlen);
int i2c_recv(unsigned int slave_addr, unsigned char *rxdata, int rxlen);
void i2c_stop(void);
void i2c_reset(void);
void i2c_siglevel(BOOL *scl, BOOL *sda);

/* ================================================
 * uart_session.c  related defines,vars,functions
 * ================================================ */

/* -------- define for uart_sesstion -------- */
#define CMDLINE_BUFSIZE 80

/* -------- global var. for uart_sesstion -------- */
extern unsigned char CMDLINE[CMDLINE_BUFSIZE];

/* -------- functions for uart_sesstion -------- */
unsigned int hex2val(unsigned char **str);
void hexstr(unsigned int val, char *str);
unsigned int num2val(unsigned char **ptr);
void numstr(unsigned int val, char *str);

void session(void);


/* ================================================
 * rtc.c  related defines,vars,functions
 * ================================================ */

/* -------- define for rtc -------- */
typedef struct {
	int year;	// 2000..2099
	char mon;	// 1..12
	char day;	// 1..31
	char hour;
	char min;
	char sec;
} DATETYPE;

/* -------- global var. for rtc -------- */
extern unsigned long RTC_TIME_COUNTER;
extern BOOL RTC_TICK;
extern BOOL RTC_FLG1;		// DEBUG,  WDT updated the counter

extern int mon_day_tbl[];

/* -------- functions for rtc -------- */
void rtc_reset(void);
int set_rtc_counter(int year, char mon, char day, char hour, char min, char sec);
DATETYPE rtc_count2ymd(unsigned long count);
void show_date_time(DATETYPE date);
int get_xcap_setting();
void set_xcap_setting(int sel);

/* ================================================
 * power_mng.c  related defines,vars,functions
 * ================================================ */

/* -------- define for power_mng -------- */
#define  PM_MODE_RUN0	10	// running, external power supply exists
#define  PM_MODE_RUN1	5	// running by self battery
#define  PM_MODE_LOW	2	// sould be enter power save mode (LPM3)
#define  PM_MODE_SLP	1	// in LPM3
#define  SLEEP_TIMER_LIMIT	10	// sec

/* -------- global var. for power_mng -------- */
extern int PM_MODE;
extern int SLEEP_TIMER;

/* -------- functions for power_mng -------- */
void init_pmmode(void);
void check_powstat();
void set_pmmode(unsigned char mode);
unsigned char cur_pmmode(void);
void any_activity(void);
void wdt_tick_pm(void);
void loop_dispatch(void);
void show_error(int code, int param1, char *mes);
void fatal_error(int code, int param1, char *mes);

/* ================================================
 * gpio.c  related defines,vars,functions
 * ================================================ */

/* -------- functions for gpio -------- */
void led_1(BOOL on);
void led_2(BOOL on);
BOOL poll_sw1(void);
BOOL poll_rxd(void);


/* ================================================
 * temp_sensor.c  related defines,vars,functions
 * ================================================ */

/* -------- define for temp_sensor -------- */
#define I2C_ADR_TEMPSNS		0x72
#define TEMPSNS_ID			0x53	// STTS751 Manufacturer ID


/* -------- global var. temp_sensor -------- */

/* -------- functions for temp_sensor -------- */
int init_temp_sensor(void);
unsigned int get_temperature(void);
void raw2degree(unsigned int rawdata, char *str);

void temp_test1();	// for DEBUG

/* ================================================
 * eeprom.c  related defines,vars,functions
 * ================================================ */

/* -------- define for eeprom -------- */
#define I2C_ADR_EEPROM		0xa0
#define EEPROM_MAX_BYTE		0x20	// max size of r/w data length

/* -------- global var. eeprom -------- */

/* -------- functions for eeprom -------- */
int eeprom_wr(unsigned int adr, unsigned char *data, unsigned int bytes);
int eeprom_rd(unsigned int adr, unsigned char *data, unsigned int bytes);

void eeprom_test1(unsigned char *param);	// for DEBUG


/* ================================================
 * rec_mng.c  related defines,vars,functions
 * ================================================ */

/* -------- define for rec_mng -------- */
#define	REC_MEM_SIZE	0x8000		// EEPROM capacity in bytes (256Mbit, 32KB)
#define REC_MODE_RUN	1		// recording
#define REC_MODE_STOP	0		// standby
#define REC_CFG_PERI_MODE_0	16		// record period: 16sec
#define REC_CFG_PERI_MODE_1	64		// record period: 64sec
#define REC_CFG_PERI_MODE_2	128		// record period: 128sec
#define REC_CFG_PERI_MODE_3	256		// record period: 256sec
#define REC_CFG_SFT_MODE_0	4		// rtc r-shift for record: 4bit
#define REC_CFG_SFT_MODE_1	4		// rtc r-shift for record: 4bit
#define REC_CFG_SFT_MODE_2	4		// rtc r-shift for record: 4bit
#define REC_CFG_SFT_MODE_3	8		// rtc r-shift for record: 8bit
#define WORK_BUFFSIZE	32			// using erase func.

#define REC_HEADER_MARK1	0xFF	// start mark 1 on eeprom
#define REC_HEADER_MARK2	0xE0
#define REC_LIST_MAX	32

/* -------- global var. rec_mng -------- */
extern int REC_MODE;					// run/stop flag
extern unsigned int REC_ITEMS;
extern unsigned char REC_CONFIG_MODE;	// config no.= 0,1,2 or 3
extern unsigned long REC_START_TIME;	// started time (RTC value)
extern unsigned int REC_START_ADR_PTR, REC_CUR_ADR_PTR;

/* -------- functions for rec_mng -------- */
void init_rec(void);
int set_rec_config(int rec_cfg);
int set_rec_mode(int rec_mode);
BOOL set_adr_ptr(unsigned int new_ptr);
void record_open(void);
void record_close(void);
void record_list_select(void);
void wdt_tick_record(void);
void show_current_rec_status(void);
void dump_record(unsigned int start_addr, unsigned int items);
void show_config_options(void);
void erase_all_memory(void);


/* ================================================
 * grace generated functions
 * ================================================ */
void System_graceInit(void);
void USCI_B0_graceInit(void);
void USCI_A0_graceInit(void);

#endif  /* _USER_DEFS_ */

